@@ -1,5 +1,4 @@
-/* $Id: irplib_framelist.c,v 1.29 2012-02-03 13:55:14 llundin Exp $
- *
+/* 
  * This file is part of the irplib package 
  * Copyright (C) 2002,2003 European Southern Observatory
  *
@@ -18,13 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1307  USA
  */
 
-/*
- * $Author: llundin $
- * $Date: 2012-02-03 13:55:14 $
- * $Revision: 1.29 $
- * $Name: not supported by cvs2svn $
- */
-
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -35,6 +27,10 @@
                                  Includes
  -----------------------------------------------------------------------------*/
 
+#include "irplib_framelist.h"
+#include "irplib_utils.h"
+#include <cpl.h>
+
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -42,10 +38,6 @@
 #include <math.h>
 #include <assert.h>
 
-#include <cpl.h>
-
-#include "irplib_framelist.h"
-
 
 /*-----------------------------------------------------------------------------
                                  New types
@@ -210,7 +202,6 @@ irplib_framelist * irplib_framelist_cast(const cpl_frameset * frameset)
 {
 
     irplib_framelist * self;
-    const cpl_frame * frame;
     int i;
 
 
@@ -219,9 +210,9 @@ irplib_framelist * irplib_framelist_cast(const cpl_frameset * frameset)
     /* The function cannot fail now */
     self = irplib_framelist_new();
 
-    for (i = 0, frame = cpl_frameset_get_first_const(frameset);
-         frame != NULL;
-         i++, frame = cpl_frameset_get_next_const(frameset)) {
+    for (i = 0; i < cpl_frameset_get_size(frameset); i++) 
+    {
+        const cpl_frame * frame = cpl_frameset_get_position_const(frameset, i);
 
         cpl_frame * copy = cpl_frame_duplicate(frame);
 
@@ -365,7 +356,7 @@ irplib_framelist * irplib_framelist_extract_regexp(const irplib_framelist* self,
     irplib_framelist * new;
     int error;
     int i, newsize;
-    const int xor = invert == CPL_FALSE ? 0 : 1;
+    const int xor_val = (invert == CPL_FALSE ? 0 : 1);
     regex_t re;
 
 
@@ -391,7 +382,7 @@ irplib_framelist * irplib_framelist_extract_regexp(const irplib_framelist* self,
         }
 
         if ((regexec(&re, tag, (size_t)0, NULL, 0) == REG_NOMATCH ? 1 : 0)
-            ^ xor) continue;
+            ^ xor_val) continue;
 
         copy = cpl_frame_duplicate(frame);
 
@@ -455,8 +446,9 @@ int irplib_framelist_get_size(const irplib_framelist * self)
 /*----------------------------------------------------------------------------*/
 cpl_frame * irplib_framelist_get(irplib_framelist * self, int pos)
 {
+    IRPLIB_DIAG_PRAGMA_PUSH_IGN(-Wcast-qual);
     return (cpl_frame *)irplib_framelist_get_const(self, pos);
-
+    IRPLIB_DIAG_PRAGMA_POP;
 }
 
 
@@ -529,9 +521,10 @@ cpl_propertylist * irplib_framelist_get_propertylist(irplib_framelist * self,
                                                      int pos)
 {
 
+    IRPLIB_DIAG_PRAGMA_PUSH_IGN(-Wcast-qual);
     return (cpl_propertylist *)irplib_framelist_get_propertylist_const(self,
                                                                        pos);
-
+    IRPLIB_DIAG_PRAGMA_POP;
 }
 
 
@@ -627,7 +620,7 @@ cpl_error_code irplib_framelist_load_propertylist(irplib_framelist * self,
   @param    invert Boolean to invert the sense of the pattern matching.
   @return   CPL_ERROR_NONE or the relevant CPL error code
   @see      irplib_framelist_load_propertylist()
-  @note     Use a regexp of "" to load all properties. If a frame already has
+  @note     Use a regexp of ".?" to load all properties. If a frame already has
             a propertylist, it is not modified (and no propertylist is loaded
             for that frame).
 
