@@ -1,7 +1,7 @@
-/* $Id: irplib_utils.c,v 1.77 2011-11-22 08:45:56 yjung Exp $
- *
+/*
  * This file is part of the irplib package
- * Copyright (C) 2002,2003 European Southern Observatory
+ * Copyright (C) 2002,2003,2014 European Southern Observatory
+ *               2004  Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1307  USA
  */
 
-/*
- * $Author: yjung $
- * $Date: 2011-11-22 08:45:56 $
- * $Revision: 1.77 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -32,27 +25,20 @@
 /*-----------------------------------------------------------------------------
                                    Includes
  -----------------------------------------------------------------------------*/
-
-#include <math.h>
+#include "irplib_utils.h"
 
 #include <string.h>
 #include <assert.h>
+#include <stdlib.h>
+#include <errno.h>
 
 #include <cpl.h>
-
-#include "irplib_utils.h"
+#include <math.h>
 
 /*-----------------------------------------------------------------------------
                            Defines
  -----------------------------------------------------------------------------*/
 
-/* TEMPORARY SUPPORT OF CPL 5.x */
-#ifndef CPL_SIZE_FORMAT
-#define CPL_SIZE_FORMAT "d"
-#define cpl_size int
-#endif
-/* END TEMPORARY SUPPORT OF CPL 5.x */
-
 #ifndef inline
 #define inline /* inline */
 #endif
@@ -61,14 +47,6 @@
                            Missing Function Prototypes
  -----------------------------------------------------------------------------*/
 
-#if defined HAVE_ISNAN && HAVE_ISNAN != 0
-#if !defined isnan && defined HAVE_DECL_ISNAN && HAVE_DECL_ISNAN == 0
-/* HP-UX and Solaris may have isnan() available at link-time
-   without the prototype */
-int isnan(double);
-#endif
-#endif
-
 /*-----------------------------------------------------------------------------
                            Private Function Prototypes
  -----------------------------------------------------------------------------*/
@@ -227,7 +205,7 @@ cpl_error_code irplib_dfs_save_image(cpl_frameset            * allframes,
     cpl_propertylist * prolist  = applist ? cpl_propertylist_duplicate(applist)
         : cpl_propertylist_new();
 
-    cpl_propertylist_append_string(prolist, CPL_DFS_PRO_CATG, procat);
+    cpl_propertylist_update_string(prolist, CPL_DFS_PRO_CATG, procat);
 
     irplib_dfs_save_image_(allframes, NULL, parlist, usedframes, NULL, image,
                            bpp, recipe, prolist, remregexp, pipe_id, filename);
@@ -273,7 +251,7 @@ irplib_dfs_save_propertylist(cpl_frameset            * allframes,
     cpl_propertylist * prolist  = applist ? cpl_propertylist_duplicate(applist)
         : cpl_propertylist_new();
 
-    cpl_propertylist_append_string(prolist, CPL_DFS_PRO_CATG, procat);
+    cpl_propertylist_update_string(prolist, CPL_DFS_PRO_CATG, procat);
 
     cpl_dfs_save_propertylist(allframes, NULL, parlist, usedframes, NULL,
                               recipe, prolist, remregexp, pipe_id, filename);
@@ -322,7 +300,7 @@ cpl_error_code irplib_dfs_save_imagelist(cpl_frameset            * allframes,
     cpl_propertylist * prolist  = applist ? cpl_propertylist_duplicate(applist)
         : cpl_propertylist_new();
 
-    cpl_propertylist_append_string(prolist, CPL_DFS_PRO_CATG, procat);
+    cpl_propertylist_update_string(prolist, CPL_DFS_PRO_CATG, procat);
 
     cpl_dfs_save_imagelist(allframes, NULL, parlist, usedframes, NULL,
                            imagelist, bpp, recipe, prolist, remregexp, pipe_id,
@@ -371,7 +349,7 @@ cpl_error_code irplib_dfs_save_table(cpl_frameset            * allframes,
     cpl_propertylist * prolist  = applist ? cpl_propertylist_duplicate(applist)
         : cpl_propertylist_new();
 
-    cpl_propertylist_append_string(prolist, CPL_DFS_PRO_CATG, procat);
+    cpl_propertylist_update_string(prolist, CPL_DFS_PRO_CATG, procat);
 
     cpl_dfs_save_table(allframes, NULL, parlist, usedframes, NULL,
                            table, tablelist, recipe, prolist, remregexp,
@@ -891,6 +869,7 @@ irplib_dfs_table_convert(cpl_table               * self,
     cpl_propertylist * applist    = NULL;
     cpl_errorstate     prestate   = cpl_errorstate_get();
     cpl_error_code     error;
+    char             * fallback_filename = NULL;
 
     cpl_ensure_code(self         != NULL, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(allframes    != NULL, CPL_ERROR_NULL_INPUT);
@@ -914,13 +893,13 @@ irplib_dfs_table_convert(cpl_table               * self,
                                      "Consistency check of table failed");
     }
 
-    filename = product_name != NULL
-        ? product_name : cpl_sprintf("%s" CPL_DFS_FITS, recipe_name);
+    fallback_filename = cpl_sprintf("%s" CPL_DFS_FITS, recipe_name);
+    filename = product_name != NULL ? product_name : fallback_filename;
 
     applist = mainlist == NULL
         ? cpl_propertylist_new() : cpl_propertylist_duplicate(mainlist);
 
-    error = cpl_propertylist_append_string(applist, "INSTRUME", instrume);
+    error = cpl_propertylist_update_string(applist, "INSTRUME", instrume);
 
     if (!error)
         error = irplib_dfs_save_table(allframes, parlist, useframes, self,
@@ -928,7 +907,7 @@ irplib_dfs_table_convert(cpl_table               * self,
                                       remregexp, pipe_id, filename);
 
     cpl_propertylist_delete(applist);
-    if (filename != product_name) cpl_free((char*)filename);
+    cpl_free(fallback_filename);
 
     /* Propagate the error, if any */
     cpl_ensure_code(!error, error);
@@ -960,7 +939,7 @@ irplib_dfs_table_convert(cpl_table               * self,
     1) How to parse the lines - each line is read with fgets().
     2) For each column: type/format (%lg/%s/%d) + label
 
-  During a succesful call self will have rows added or removed to exactly
+  During a successful call self will have rows added or removed to exactly
   match the number of lines converted. Any a priori knowledge about the
   expected number of converted rows can be used in the creation of the
   table (to reduce memory reallocation overhead).
@@ -1009,6 +988,7 @@ irplib_table_read_from_frameset(cpl_table               * self,
     int                nrow       = cpl_table_get_nrow(self);
     int                irow       = 0;
     cpl_errorstate     prestate   = cpl_errorstate_get();
+    cpl_frameset_iterator * iterator = NULL;
 
     cpl_ensure_code(self         != NULL, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(useframes    != NULL, CPL_ERROR_NULL_INPUT);
@@ -1018,9 +998,9 @@ irplib_table_read_from_frameset(cpl_table               * self,
 
     linebuffer = cpl_malloc(maxlinelen);
 
-    for (rawframe = cpl_frameset_get_first_const(useframes);
+    for (rawframe = irplib_frameset_get_first_const(&iterator, useframes);
          rawframe != NULL;
-         rawframe = cpl_frameset_get_next_const(useframes), nfiles++) {
+         rawframe = irplib_frameset_get_next_const(iterator), nfiles++) {
 
         const char * rawfile = cpl_frame_get_filename(rawframe);
         const char * done; /* Indicate when the reading is done */
@@ -1098,6 +1078,7 @@ irplib_table_read_from_frameset(cpl_table               * self,
                             1+nfiles, rawfile);
     }
 
+    cpl_frameset_iterator_delete(iterator);
     cpl_free(linebuffer);
     if (stream != NULL) fclose(stream);
 
@@ -1153,15 +1134,15 @@ int irplib_compare_tags(
         cpl_frame   *   frame1,
         cpl_frame   *   frame2)
 {
-    char            *   v1 ;
-    char            *   v2 ;
+    const char            *   v1 ;
+    const char            *   v2 ;
 
     /* Test entries */
     if (frame1==NULL || frame2==NULL) return -1 ;
 
     /* Get the tags */
-    if ((v1 = (char*)cpl_frame_get_tag(frame1)) == NULL) return -1 ;
-    if ((v2 = (char*)cpl_frame_get_tag(frame2)) == NULL) return -1 ;
+    if ((v1 = cpl_frame_get_tag(frame1)) == NULL) return -1 ;
+    if ((v2 = cpl_frame_get_tag(frame2)) == NULL) return -1 ;
 
     /* Compare the tags */
     if (strcmp(v1, v2)) return 0 ;
@@ -1220,13 +1201,16 @@ cpl_frame * irplib_frameset_get_first_from_group(const cpl_frameset * self,
                                                  cpl_frame_group      group)
 {
     const cpl_frame * frame;
+    cpl_frameset_iterator * iterator = NULL;
 
     cpl_ensure(self != NULL, CPL_ERROR_NULL_INPUT, NULL);
 
-    for (frame = cpl_frameset_get_first_const(self); frame != NULL ;
-         frame = cpl_frameset_get_next_const(self)) {
+    for (frame = irplib_frameset_get_first_const(&iterator, self);
+         frame != NULL ;
+         frame = irplib_frameset_get_next_const(iterator)) {
         if (cpl_frame_get_group(frame) == group) break;
     }
+    cpl_frameset_iterator_delete(iterator);
     return frame;
 }
 
@@ -1289,87 +1273,6 @@ cpl_error_code irplib_apertures_find_max_flux(const cpl_apertures * self,
 
 }
 
-/*----------------------------------------------------------------------------*/
-/**
-   @brief   portable isinf
- */
-/*----------------------------------------------------------------------------*/
-int irplib_isinf(double value)
-{
-#if defined HAVE_ISINF && HAVE_ISINF
-    return isinf(value);
-#else
-    return value != 0 && value == 2 * value;
-#endif
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-   @brief   portable isnan
- */
-/*----------------------------------------------------------------------------*/
-int irplib_isnan(double value)
-{
-#if defined HAVE_ISNAN && HAVE_ISNAN
-    return isnan(value);
-#else
-    return value != value;
-#endif
-}
-
-
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    Dump a single CPL error
-  @param    self      The number of the current error to be dumped
-  @param    first     The number of the first error to be dumped
-  @param    last      The number of the last error to be dumped
-  @return   void
-  @note This function differs from cpl_errorstate_dump_one only in message level
-  @see cpl_errorstate_dump_one
-
- */
-/*----------------------------------------------------------------------------*/
-void irplib_errorstate_warning(unsigned self, unsigned first, unsigned last)
-{
-
-    const cpl_boolean is_reverse = first > last ? CPL_TRUE : CPL_FALSE;
-    const unsigned    newest     = is_reverse ? first : last;
-    const unsigned    oldest     = is_reverse ? last : first;
-    const char      * revmsg     = is_reverse ? " in reverse order" : "";
-
-
-    assert( oldest <= self );
-    assert( newest >= self );
-
-    if (newest == 0) {
-        cpl_msg_info(cpl_func, "No error(s) to dump");
-        assert( oldest == 0);
-    } else {
-        assert( oldest > 0);
-        assert( newest >= oldest);
-        if (self == first) {
-            if (oldest == 1) {
-                cpl_msg_warning(cpl_func, "Dumping all %u error(s)%s:", newest,
-                              revmsg);
-            } else {
-                cpl_msg_warning(cpl_func, "Dumping the %u most recent error(s) "
-                              "out of a total of %u errors%s:",
-                              newest - oldest + 1, newest, revmsg);
-            }
-            cpl_msg_indent_more();
-        }
-
-        cpl_msg_warning(cpl_func, "[%u/%u] '%s' (%u) at %s", self, newest,
-                      cpl_error_get_message(), cpl_error_get_code(),
-                      cpl_error_get_where());
-
-        if (self == last) cpl_msg_indent_less();
-    }
-}
-
-
 /**@}*/
 
 
@@ -1554,27 +1457,27 @@ static cpl_polynomial * irplib_polynomial_fit_1d_create_common(
     x_size = cpl_vector_get_size(x_pos);    
     if(fit1d != NULL && x_size > 1)
     {
-      cpl_matrix     * samppos = NULL;
-      cpl_vector     * fitresidual = NULL;
-		cpl_ensure(!cpl_error_get_code(), cpl_error_get_code(), NULL);
-		samppos = cpl_matrix_wrap(1, x_size,
-												   (double*)cpl_vector_get_data_const(x_pos));
-		cpl_ensure(!cpl_error_get_code(), cpl_error_get_code(), NULL);
-		fitresidual = cpl_vector_new(x_size);
-		cpl_ensure(!cpl_error_get_code(), cpl_error_get_code(), NULL);
-		cpl_polynomial_fit(fit1d, samppos, NULL, values, NULL,
-						   CPL_FALSE, NULL, &loc_degree);
-		cpl_ensure(!cpl_error_get_code(), cpl_error_get_code(), NULL);
-		cpl_vector_fill_polynomial_fit_residual(fitresidual, values, NULL, fit1d,
-												samppos, rechisq);
-		cpl_ensure(!cpl_error_get_code(), cpl_error_get_code(), NULL);
-		if (mse)
-		{
-			*mse = cpl_vector_product(fitresidual, fitresidual)
-				/ cpl_vector_get_size(fitresidual);
-		}
-		cpl_matrix_unwrap(samppos);
-		cpl_vector_delete(fitresidual);
+        cpl_matrix     * samppos = NULL;
+        cpl_vector     * fitresidual = NULL;
+        cpl_ensure(!cpl_error_get_code(), cpl_error_get_code(), NULL);
+        samppos = cpl_matrix_wrap(1, x_size,
+                                  (double*)cpl_vector_get_data_const(x_pos));
+        cpl_ensure(!cpl_error_get_code(), cpl_error_get_code(), NULL);
+        fitresidual = cpl_vector_new(x_size);
+        cpl_ensure(!cpl_error_get_code(), cpl_error_get_code(), NULL);
+        cpl_polynomial_fit(fit1d, samppos, NULL, values, NULL,
+                           CPL_FALSE, NULL, &loc_degree);
+        cpl_ensure(!cpl_error_get_code(), cpl_error_get_code(), NULL);
+        cpl_vector_fill_polynomial_fit_residual(fitresidual, values, NULL,
+                                                fit1d, samppos, rechisq);
+        cpl_ensure(!cpl_error_get_code(), cpl_error_get_code(), NULL);
+        if (mse)
+        {
+            *mse = cpl_vector_product(fitresidual, fitresidual)
+                / cpl_vector_get_size(fitresidual);
+        }
+        cpl_matrix_unwrap(samppos);
+        cpl_vector_delete(fitresidual);
     }
     return fit1d;
 }
@@ -1616,21 +1519,22 @@ static void quicksort(int* iindex, double* exptime, int left, int right)
 }
 cpl_error_code irplib_frameset_sort(const cpl_frameset *  self, int* iindex, double* exptime)
 {
-	int sz = 0;
 	int i = 0;
 	const cpl_frame* tmp_frame = 0;
 	cpl_error_code error = CPL_ERROR_NONE;
-	sz = cpl_frameset_get_size(self);
+	int sz = cpl_frameset_get_size(self);
+	cpl_frameset_iterator* iterator = NULL;
 
 	/* 1. get an array of frames */
-	tmp_frame = cpl_frameset_get_first_const(self);
+	tmp_frame = irplib_frameset_get_first_const(&iterator, self);
 	while(tmp_frame)
 	{
 		exptime[i] = frame_get_exptime(tmp_frame);
 		iindex[i] = i;
-		tmp_frame = cpl_frameset_get_next_const(self);
+		tmp_frame = irplib_frameset_get_next_const(iterator);
 		i++;
 	}
+	cpl_frameset_iterator_delete(iterator);
 	/* 2.sort */
 	quicksort(iindex, exptime, 0, sz - 1);
 
@@ -1639,21 +1543,186 @@ cpl_error_code irplib_frameset_sort(const cpl_frameset *  self, int* iindex, dou
 
 static double frame_get_exptime(const cpl_frame * pframe)
 {
-    cpl_propertylist       *plist = 0;
-    double                  dval = 0;
-
-    plist = cpl_propertylist_load(cpl_frame_get_filename(pframe),0);
-	if(plist)
-	{
-      cpl_error_code err = CPL_ERROR_NONE;  
-	    dval = cpl_propertylist_get_double(plist, "EXPTIME");
-       err = cpl_error_get_code();
-       if (err != CPL_ERROR_NONE)
-       {
-         cpl_msg_error(cpl_func, "error during reading EXPTIME key from the frame [%s]", cpl_frame_get_filename(pframe));
-       }
-	}
-	/* Free and return */
-	cpl_propertylist_delete(plist);
-	return dval;
+    double dval = 0;
+    cpl_propertylist * plist =
+        cpl_propertylist_load_regexp(cpl_frame_get_filename(pframe), 0,
+                                     "EXPTIME", CPL_FALSE);
+    if(plist) {
+        dval = cpl_propertylist_get_double(plist, "EXPTIME");
+        if (cpl_error_get_code() != CPL_ERROR_NONE) {
+            cpl_msg_error(cpl_func, "error during reading EXPTIME key from "
+                          "the frame [%s]", cpl_frame_get_filename(pframe));
+        }
+    }
+    /* Free and return */
+    cpl_propertylist_delete(plist);
+    return dval;
+}
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    allocate uninitialized aligned memory
+  @param    alignment alignment of the data, must be a power of two
+  @param    size      size of the memory block to be allocated in bytes
+  @return   pointer to aligned memory or on failure returns NULL and sets errno
+  @see irplib_aligned_free
+  @note memory MUST be free'd with irplib_aligned_free and cannot be realloc'd
+        memory leaks will not be detected by cpl
+        It is recommended to build with posix 2001 to get posix_memalign and add
+        AC_CHECK_FUNCS([posix_memalign]) and AC_CHECK_DECLS([posix_memalign]) to
+        configure.ac
+
+ */
+/*----------------------------------------------------------------------------*/
+void * irplib_aligned_malloc(size_t alignment, size_t size)
+{
+    if (alignment == 0)
+        alignment = 1;
+    /* Error if align is not a power of two.  */
+    if (alignment & (alignment - 1)) {
+        errno = EINVAL;
+        return NULL;
+    }
+    /* make size a multiple of alignment (required by C11) */
+    if ((size % alignment) != 0) {
+        size += alignment - (size % alignment);
+    }
+
+#if defined HAVE_DECL_ALIGNED_ALLOC && defined HAVE_ALIGNED_ALLOC
+    return aligned_alloc(alignment, size);
+#elif defined HAVE_POSIX_MEMALIGN && defined HAVE_DECL_POSIX_MEMALIGN
+    {
+        void *ptr;
+        if (alignment == 1)
+            return malloc (size);
+        if (alignment == 2 || (sizeof (void *) == 8 && alignment == 4))
+            alignment = sizeof (void *);
+        if (posix_memalign (&ptr, alignment, size) == 0)
+            return ptr;
+        else
+            return NULL;
+    }
+#else
+    /* copied from gmm_malloc.h in gcc-4.8 */
+    {
+        void * malloc_ptr;
+        void * aligned_ptr;
+
+        if (size == 0)
+            return NULL;
+
+        /* Assume malloc'd pointer is aligned at least to sizeof (void*).
+           If necessary, add another sizeof (void*) to store the value
+           returned by malloc. Effectively this enforces a minimum alignment
+           of sizeof double. */
+        if (alignment < 2 * sizeof (void *))
+            alignment = 2 * sizeof (void *);
+
+        malloc_ptr = malloc (size + alignment);
+        if (!malloc_ptr)
+            return NULL;
+
+        /* Align  We have at least sizeof (void *) space below malloc'd ptr. */
+        aligned_ptr = (void *) (((size_t) malloc_ptr + alignment)
+                                & ~((size_t) (alignment) - 1));
+
+        /* Store the original pointer just before p.  */
+        *(((void **) aligned_ptr) - 1) = malloc_ptr;
+
+        return aligned_ptr;
+    }
+#endif
+}
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    allocate aligned memory initialized to zero
+  @param    alignment alignment of the data, must be a power of two
+  @param    nelem     number of elements in buffer
+  @param    nbytes    size of single element in bytes
+  @return   pointer to aligned memory or on failure returns NULL and sets errno
+  @see irplib_aligned_malloc
+
+ */
+/*----------------------------------------------------------------------------*/
+void * irplib_aligned_calloc(size_t alignment, size_t nelem, size_t nbytes)
+{
+    void * buffer = irplib_aligned_malloc(alignment, nelem * nbytes);
+    if (buffer == NULL)
+        return NULL;
+    /* cast to aligned pointer helps compilers to emit better (builtin) code */
+    memset((size_t *)buffer, 0, nelem * nbytes);
+    return buffer;
+}
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    free aligned memory
+  @param    aligned_ptr    pointer to memory to be free'd
+  @see irplib_aligned_malloc
+  @note memory MUST be allocated with irplib_aligned_[cm]alloc
+
+ */
+/*----------------------------------------------------------------------------*/
+void irplib_aligned_free (void * aligned_ptr)
+{
+#if defined HAVE_DECL_ALIGNED_ALLOC && defined HAVE_ALIGNED_ALLOC
+  free(aligned_ptr);
+#elif defined HAVE_POSIX_MEMALIGN && defined HAVE_DECL_POSIX_MEMALIGN
+  free(aligned_ptr);
+#else
+  if (aligned_ptr)
+      free (*(((void **) aligned_ptr) - 1));
+#endif
+}
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief Return the first frame in a frameset using the iterator API.
+  @param[out]  iterator  Location of frameset iterator pointer that will be
+        filled with a new iterator instance for subsequent calls to the function
+        @c irplib_frameset_get_next_const.
+  @param[in]   frameset  The frameset to iterate over.
+  @return The first frame in the frameset or NULL on error.
+  @note The iterator instance returned in the @a iterator pointer must be
+        cleaned up with a call to @c cpl_frameset_iterator_delete. If an error
+        occurred and a new iterator instance was not created then @c *iterator
+        will be set to NULL and need not be cleaned up.
+ */
+/*----------------------------------------------------------------------------*/
+const cpl_frame *
+irplib_frameset_get_first_const(cpl_frameset_iterator **iterator,
+                                const cpl_frameset *frameset)
+{
+    cpl_ensure(iterator != NULL, CPL_ERROR_NULL_INPUT, NULL);
+    *iterator = cpl_frameset_iterator_new(frameset);
+    return cpl_frameset_iterator_get_const(*iterator);
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief Return the next frame in a frameset using the iterator API.
+  @param[in]  iterator  Iterator instance that was returned in an initial call
+                        to @c _irplib_frameset_get_first_const.
+  @return The next frame in the frameset or NULL if already at the end of the
+          list. NULL is also returned if an error occurred which can be checked
+          with a call to @c cpl_error_get_code.
+ */
+/*----------------------------------------------------------------------------*/
+const cpl_frame *
+irplib_frameset_get_next_const(cpl_frameset_iterator *iterator)
+{
+    cpl_errorstate prestate = cpl_errorstate_get();
+    cpl_error_code error = cpl_frameset_iterator_advance(iterator, 1);
+    if (error == CPL_ERROR_ACCESS_OUT_OF_RANGE) {
+        cpl_errorstate_set(prestate);
+        return NULL;
+    } else if (error != CPL_ERROR_NONE) {
+        return NULL;
+    }
+    return cpl_frameset_iterator_get_const(iterator);
 }
