@@ -1,4 +1,4 @@
-/* $Id: visir_img_dark.c,v 1.28 2011/12/21 14:42:52 llundin Exp $
+/* $Id: visir_img_dark.c,v 1.28 2011-12-21 14:42:52 llundin Exp $
  *
  * This file is part of the VISIR Pipeline
  * Copyright (C) 2002,2003 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/12/21 14:42:52 $
+ * $Date: 2011-12-21 14:42:52 $
  * $Revision: 1.28 $
- * $Name: visir-3_5_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -320,19 +320,14 @@ static cpl_error_code visir_img_dark_reduce(cpl_propertylist * qclist,
 
     skip_if (nfiles < 2);
 
-    skip_if (irplib_framelist_contains(f_one, "NAXIS1",
-                                       CPL_TYPE_INT, CPL_TRUE, 0.0));
-   
-    skip_if (irplib_framelist_contains(f_one, "NAXIS2",
-                                       CPL_TYPE_INT, CPL_TRUE, 0.0));
- 
     for (i=0 ; i < nfiles ; i++) {
         const cpl_frame        * frame = irplib_framelist_get_const(f_one, i);
         const char             * name  = cpl_frame_get_filename(frame);
+        cpl_size next                  = cpl_frame_get_nextensions(frame);
 
         cpl_image_delete(diff);
         diff = dark;
-        irplib_check(dark = cpl_image_load(name, CPL_TYPE_FLOAT, 0, 0),
+        irplib_check(dark = cpl_image_load(name, CPL_TYPE_FLOAT, 0, next),
                      "Could not load FITS-image from %s", name);
 
         if (i == 0) {
