@@ -1,4 +1,4 @@
-/* $Id: visir_util_join.c,v 1.2 2011/07/28 15:02:10 llundin Exp $
+/* $Id: visir_util_join.c,v 1.13 2013-05-13 16:03:43 jtaylor Exp $
  *
  * This file is part of the VISIR Pipeline
  * Copyright (C) 2011 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2011/07/28 15:02:10 $
- * $Revision: 1.2 $
- * $Name: visir-3_5_1 $
+ * $Author: jtaylor $
+ * $Date: 2013-05-13 16:03:43 $
+ * $Revision: 1.13 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,6 +34,7 @@
  -----------------------------------------------------------------------------*/
 
 #include "visir_recipe.h"
+#include <string.h>
 
 /*-----------------------------------------------------------------------------
                                 Defines
@@ -52,10 +53,15 @@ cpl_error_code visir_util_join_one(cpl_frameset *,
                                    const irplib_framelist *,
                                    const irplib_framelist *,
                                    const irplib_framelist *,
+                                   const irplib_framelist *,
                                    int, cpl_boolean, cpl_boolean, 
                                    cpl_boolean, cpl_boolean,
+                                   cpl_boolean,
                                    const cpl_parameterlist *);
 
+#ifdef VISIR_CHAIN
+#define cpl_plugin_get_info visir_util_join_get_info
+#endif
 VISIR_RECIPE_DEFINE(visir_util_join, 0,
                     "Extend the final product with extensions containing its "
                     "error map, bad pixel map and weight or contribution map",
@@ -96,28 +102,36 @@ VISIR_RECIPE_DEFINE(visir_util_join, 0,
 static int visir_util_join(cpl_frameset            * framelist,
                            const cpl_parameterlist * parlist)
 {
-#ifdef _OPENMP
-    cpl_errorstate     cleanstate = cpl_errorstate_get();
-#endif
-    cpl_error_code     didfail   = CPL_ERROR_NONE;
     irplib_framelist * allframes = NULL;
     irplib_framelist * rawframes = NULL;
     irplib_framelist * bpmframes = NULL;
     irplib_framelist * errframes = NULL;
     irplib_framelist * conframes = NULL;
     irplib_framelist * wgtframes = NULL;
-    int                i, n;
-    int                nbad = 0, nerr = 0, ncon = 0, nwgt = 0;
+    irplib_framelist * qcframes = NULL;
+    int                n;
+    int                nbad = 0, nerr = 0, ncon = 0, nwgt = 0, nqc = 0;
     
 
     /* Identify the RAW and TAB frames in the input frameset */
     skip_if (visir_dfs_set_groups(framelist));
+    FOR_EACH_FRAMESET(frm, framelist)
+        cpl_frame_set_group(frm, CPL_FRAME_GROUP_RAW);
 
     /* Objects observation */
     allframes = irplib_framelist_cast(framelist);
     skip_if(allframes == NULL);
 
-    rawframes = irplib_framelist_extract(allframes, VISIR_UTIL_DATA);
+    rawframes =
+        irplib_framelist_extract_regexp(allframes, "^("
+                                        VISIR_UTIL_DATA
+                                        "|" VISIR_IMG_PHOT_ONEBEAM_PROCATG
+                                        "|" VISIR_IMG_PHOT_COMBINED_PROCATG
+                                        "|COADDED_IMAGE"
+                                        "|COADDED_IMAGE_COMBINED"
+                                        ")$",
+                                        CPL_FALSE);
+
     skip_if(rawframes == NULL);
 
     n = irplib_framelist_get_size(rawframes);
@@ -131,9 +145,9 @@ static int visir_util_join(cpl_frameset            * framelist,
                  "%d raw-frames <=> %d bpm frames", n, nbad);
     }
 
-    if (cpl_frameset_find_const(framelist, VISIR_UTIL_WEIGHT2ERROR_PROCATG)) {
+    if (cpl_frameset_find_const(framelist, VISIR_UTIL_ERROR_MAP_PROCATG)) {
         errframes = irplib_framelist_extract(allframes,
-                                             VISIR_UTIL_WEIGHT2ERROR_PROCATG);
+                                             VISIR_UTIL_ERROR_MAP_PROCATG);
         skip_if (errframes == NULL);
 
         nerr = irplib_framelist_get_size(errframes);
@@ -152,9 +166,9 @@ static int visir_util_join(cpl_frameset            * framelist,
                  "%d raw-frames <=> %d contribution frames", n, ncon);
     }
 
-    if (cpl_frameset_find_const(framelist, VISIR_UTIL_WEIGHT2ERROR)) {
+    if (cpl_frameset_find_const(framelist, VISIR_UTIL_WEIGHT_MAP_PROCATG)) {
         wgtframes = irplib_framelist_extract(allframes,
-                                             VISIR_UTIL_WEIGHT2ERROR);
+                                             VISIR_UTIL_WEIGHT_MAP_PROCATG);
         skip_if (wgtframes == NULL);
 
         nwgt = irplib_framelist_get_size(wgtframes);
@@ -162,39 +176,23 @@ static int visir_util_join(cpl_frameset            * framelist,
                  "%d raw-frames <=> %d weight frames", n, nwgt);
     }
 
-#ifdef _OPENMP
-#pragma omp parallel for private(i)
-#endif
-    for (i = 0; i < n; i++) {
-        if (!didfail) {
+    if (cpl_frameset_find_const(framelist, VISIR_UTIL_QC_PROCATG)) {
+        qcframes = irplib_framelist_extract(allframes, VISIR_UTIL_QC_PROCATG);
+        skip_if (qcframes == NULL);
 
-            /* The total number of iterations must be pre-determined for the
-               parallelism to work. In case of an error we can therefore not
-               break, so instead we skip immediately to the next iteration.
-               FIXME: This check on didfail does not guarantee that only one
-               iteration can cause an error to be dumped, but it is not
-               worse than checking on a thread-local state, e.g. errori. */
-
-            cpl_msg_info(cpl_func, "Joining frame %d/%d", 1+i, n);
-
-            if (visir_util_join_one(framelist, rawframes, bpmframes, errframes,
-                                    conframes, wgtframes, i, nbad == 1,
-                                    nerr == 1, ncon == 1, nwgt == 1,
-                                    parlist)) {
-            const cpl_error_code errori = cpl_error_set_where(cpl_func);
-#ifdef _OPENMP
-                /* Cannot access these errors after the join,
-                   so dump them now. :-(((((((((((((((((((( */
-                cpl_errorstate_dump(cleanstate, CPL_FALSE, NULL);
-                cpl_errorstate_set(cleanstate);
-#pragma omp critical(visir_util_join)
-#endif
-                didfail = errori;
-            }
-        }
+        nqc = irplib_framelist_get_size(qcframes);
+        error_if(nqc != n && nqc != 1, CPL_ERROR_INCOMPATIBLE_INPUT,
+                 "%d raw-frames <=> %d qc frames", n, nqc);
     }
 
-    error_if(didfail, didfail, "Failed to process %d frame(s)", n);
+    for (int i = 0; i < n; i++) {
+        cpl_msg_info(cpl_func, "Joining frame %d/%d", 1+i, n);
+
+        skip_if (visir_util_join_one(framelist, rawframes, bpmframes, errframes,
+                                     conframes, wgtframes, qcframes, i, nbad == 1,
+                                     nerr == 1, ncon == 1, nwgt == 1, nqc == 1,
+                                     parlist));
+    }
 
     end_skip;
 
@@ -204,6 +202,7 @@ static int visir_util_join(cpl_frameset            * framelist,
     irplib_framelist_delete(errframes);
     irplib_framelist_delete(conframes);
     irplib_framelist_delete(wgtframes);
+    irplib_framelist_delete(qcframes);
 
     return cpl_error_get_code();
 }
@@ -218,11 +217,13 @@ static int visir_util_join(cpl_frameset            * framelist,
   @param    errframes   The error frames list
   @param    conframes   The contribution frames list
   @param    wgtframes   The weight frames list
+  @param    qcframes    The qc parameter frames list
   @param    i           The frame to process (0 for first)
   @param    bshared     CPL_TRUE iff the bpm is shared among all raw frames
   @param    eshared     CPL_TRUE iff the error is shared among all raw frames
   @param    cshared     CPL_TRUE iff the contrib is shared among all raw frames
   @param    wshared     CPL_TRUE iff the weight is shared among all raw frames
+  @param    qcshared     CPL_TRUE iff the qc list is shared among all raw frames
   @param    parlist     The parameters list
   @return   CPL_ERROR_NONE iff OK, otherwise the relevant CPL error code.
  */
@@ -234,9 +235,11 @@ cpl_error_code visir_util_join_one(cpl_frameset * framelist,
                                    const irplib_framelist * errframes,
                                    const irplib_framelist * conframes,
                                    const irplib_framelist * wgtframes,
+                                   const irplib_framelist * qcframes,
                                    int i,
                                    cpl_boolean bshared, cpl_boolean eshared,
                                    cpl_boolean cshared, cpl_boolean wshared,
+                                   cpl_boolean qcshared,
                                    const cpl_parameterlist * parlist)
 {
 
@@ -245,78 +248,86 @@ cpl_error_code visir_util_join_one(cpl_frameset * framelist,
     cpl_frameset     * products   = cpl_frameset_new();
     cpl_frameset     * usedframes = cpl_frameset_new();
     const char       * filename;
-    cpl_propertylist * plist      = NULL;
-    char             * proname    = cpl_sprintf(RECIPE_STRING "_%d"
-                                                CPL_DFS_FITS, 1+i);
-    cpl_image        * image      = NULL;
+    cpl_propertylist * img_plist  = NULL;
+    char             * proname    = NULL;
+    cpl_image        * img        = NULL;
     cpl_image        * csum       = NULL;
-    cpl_mask         * bpm        = NULL; 
+    const char       * bunit      = NULL;
+
+    cpl_image * bpm = NULL;
+    cpl_image * err = NULL;
+    cpl_image * wgt = NULL;
+    cpl_image * con = NULL;
+
+    cpl_propertylist * bpm_plist = NULL;
+    cpl_propertylist * err_plist = NULL;
+    cpl_propertylist * wgt_plist = NULL;
+    cpl_propertylist * con_plist = NULL;
+    cpl_propertylist * qc_plist  = NULL;
+
+    const char * procatg = VISIR_UTIL_JOIN_PROCATG;
 
     bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate
                                (irplib_framelist_get_const(rawframes, i))));
 
-    if (bpmframes != NULL) {
-        frame = irplib_framelist_get_const(bpmframes, bshared ? 0 : i);
-        bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate(frame)));
-    }
-
-    if (errframes != NULL) {
-        frame = irplib_framelist_get_const(errframes, eshared ? 0 : i);
-        bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate(frame)));
-    }
-
-    if (conframes != NULL && (irplib_framelist_get_size(conframes) == n ||
-                              cshared)) {
-        frame = irplib_framelist_get_const(conframes, cshared ? 0 : i);
-        bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate(frame)));
-    }
-
-    if (wgtframes != NULL) {
-        frame = irplib_framelist_get_const(wgtframes, wshared ? 0 : i);
-        bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate(frame)));
-    }
-
     /* 1) The data frame */
     frame = irplib_framelist_get_const(rawframes, i);
     filename = cpl_frame_get_filename(frame);
 
-    plist = cpl_propertylist_load(filename, 0);
-    skip_if(plist == NULL);
+    if (!strcmp(cpl_frame_get_tag(frame), VISIR_IMG_PHOT_ONEBEAM_PROCATG))
+        procatg = VISIR_IMG_PHOT_ONEBEAM_PROCATG;
+    else if (!strcmp(cpl_frame_get_tag(frame), VISIR_IMG_PHOT_COMBINED_PROCATG))
+        procatg = VISIR_IMG_PHOT_COMBINED_PROCATG;
+    else if (!strcmp(cpl_frame_get_tag(frame), "COADDED_IMAGE_COMBINED"))
+        procatg = "IMG_OBJ_COMBINED";
+    else if (!strcmp(cpl_frame_get_tag(frame), "COADDED_IMAGE"))
+        procatg = "IMG_OBJ_ONEBEAM";
 
-    /* FIXME: Change to CPL_TYPE_UNSPECIFIED when supported */
-    image = cpl_image_load(filename, CPL_TYPE_FLOAT, 0, 0);
-    skip_if(image == NULL);
+    img_plist = cpl_propertylist_load(filename, 0);
+    skip_if(img_plist == NULL);
 
-    /* FIXME: Change to CPL_TYPE_UNSPECIFIED when supported */
-    skip_if(irplib_dfs_save_image(products, parlist, usedframes,
-                                  image, CPL_BPP_IEEE_FLOAT,
-                                  RECIPE_STRING,
-                                  VISIR_UTIL_JOIN_PROCATG, plist,
-                                  NULL, visir_pipe_id, proname));
+    img = cpl_image_load(filename, CPL_TYPE_UNSPECIFIED, 0, 0);
+    skip_if(img == NULL);
+
+    if (qcframes) {
+        frame = irplib_framelist_get_const(qcframes, qcshared ? 0 : i);
+        filename = cpl_frame_get_filename(frame);
+        qc_plist = cpl_propertylist_load_regexp(filename, 0, "ESO QC |BUNIT", CPL_FALSE);
+        skip_if (qc_plist == NULL);
+        if (cpl_propertylist_has(qc_plist, "BUNIT")) {
+            bunit = cpl_propertylist_get_string(qc_plist, "BUNIT");
+        }
+    }
+
+    if (bunit == NULL && cpl_propertylist_has(img_plist, "BUNIT")) {
+        bunit = cpl_propertylist_get_string(img_plist, "BUNIT");
+    }
+    cpl_propertylist_append(img_plist, qc_plist);
+
+
+    if (cpl_propertylist_has(img_plist, "ESO QC BEAMID")) {
+        const char * b = cpl_propertylist_get_string(img_plist,
+                                                     "ESO QC BEAMID");
+        proname = cpl_sprintf(RECIPE_STRING "_b%s_%03d"
+                              CPL_DFS_FITS, b, 1+i);
+    }
+    else
+        proname = cpl_sprintf(RECIPE_STRING "_%03d"
+                              CPL_DFS_FITS, 1+i);
 
     if (bpmframes != NULL) {
-        /* 2) The bpm frame */
-
+        /* 3) The bpm frame */
         frame = irplib_framelist_get_const(bpmframes, bshared ? 0 : i);
 
         filename = cpl_frame_get_filename(frame);
 
-        cpl_propertylist_delete(plist);
-        plist = cpl_propertylist_load_regexp(filename, 0, " ESO QC | ESO PRO ",
-                                             0);
-        skip_if(plist == NULL);
+        bpm = cpl_image_load(filename, CPL_TYPE_INT, 0, 0);
+        skip_if(bpm == NULL);
 
-        bug_if(cpl_propertylist_append_string(plist, "EXTNAME",
-                                              "BAD PIXEL MAP"));
-
-        /* FIXME: Change to CPL_TYPE_UNSPECIFIED when supported */
-        cpl_image_delete(image);
-        image = cpl_image_load(filename, CPL_TYPE_INT, 0, 0);
-        skip_if(image == NULL);
-
-        /* FIXME: Change to CPL_TYPE_UNSPECIFIED when supported */
-        skip_if(cpl_image_save(image, proname, CPL_BPP_8_UNSIGNED,
-                               plist, CPL_IO_EXTEND));
+        bpm_plist = cpl_propertylist_load_regexp(filename, 0, " ESO QC | ESO PRO ",
+                                                 0);
+        skip_if(bpm_plist == NULL);
+        cpl_propertylist_append_string(bpm_plist, "EXTNAME", VISIR_EXTN_BPM);
     }
 
     if (errframes != NULL) {
@@ -326,41 +337,31 @@ cpl_error_code visir_util_join_one(cpl_frameset * framelist,
 
         filename = cpl_frame_get_filename(frame);
 
-        cpl_propertylist_delete(plist);
-        plist = cpl_propertylist_load_regexp(filename, 0, " ESO QC | ESO PRO ",
+        err = cpl_image_load(filename, CPL_TYPE_UNSPECIFIED, 0, 0);
+        skip_if(err == NULL);
+
+        err_plist = cpl_propertylist_load_regexp(filename, 0, " ESO QC | ESO PRO ",
                                              0);
-        skip_if(plist == NULL);
+        skip_if(err_plist == NULL);
+        cpl_propertylist_append_string(err_plist, "EXTNAME", VISIR_EXTN_ERROR);
+        if (bunit) {
+            cpl_propertylist_append_string(err_plist, "BUNIT", bunit);
+        }
 
-        bug_if(cpl_propertylist_append_string(plist, "EXTNAME", "ERROR MAP"));
-
-        /* FIXME: Change to CPL_TYPE_UNSPECIFIED when supported */
-        cpl_image_delete(image);
-        image = cpl_image_load(filename, CPL_TYPE_FLOAT, 0, 0);
-        skip_if(image == NULL);
-
-        /* FIXME: Change to CPL_TYPE_UNSPECIFIED when supported */
-        skip_if(cpl_image_save(image, proname, CPL_BPP_IEEE_FLOAT,
-                               plist, CPL_IO_EXTEND));
-
-        if (conframes == NULL && bpmframes == NULL) {
-
-            cpl_propertylist_empty(plist);
-            bug_if(cpl_propertylist_append_string(plist, "EXTNAME",
-                                                  "BAD PIXEL MAP"));
-
-            /* FIXME: Find a better way to create a image BPM */
-            bpm = cpl_mask_threshold_image_create(image, -DBL_EPSILON, 1e8);
-            cpl_mask_not(bpm);
-            cpl_image_delete(image);
-            image = cpl_image_new_from_mask(bpm);
-
-            skip_if(cpl_image_save(image, proname, CPL_BPP_8_UNSIGNED,
-                                   plist, CPL_IO_EXTEND));
+        if (wgtframes == NULL) {
+            wgt = cpl_image_new(cpl_image_get_size_x(err),
+                                cpl_image_get_size_y(err),
+                                CPL_TYPE_FLOAT);
+            cpl_image_add_scalar(wgt, 1.0);
+            cpl_image_divide(wgt, err);
+            cpl_image_power(wgt, 2);
+            wgt_plist = cpl_propertylist_new();
+            cpl_propertylist_append_string(wgt_plist, "EXTNAME", VISIR_EXTN_WEIGHT);
+            skip_if(0);
         }
 
     }
 
-
     if (conframes != NULL) {
         /* 4) The contribution frame(s) */
 
@@ -370,15 +371,12 @@ cpl_error_code visir_util_join_one(cpl_frameset * framelist,
 
             filename = cpl_frame_get_filename(frame);
 
-            cpl_propertylist_delete(plist);
-            plist = cpl_propertylist_load_regexp(filename, 0,
-                                                 " ESO QC | ESO PRO ", 0);
-            skip_if(plist == NULL);
+            con = cpl_image_load(filename, CPL_TYPE_INT, 0, 0);
+            skip_if(con == NULL);
 
-            /* FIXME: Change to CPL_TYPE_UNSPECIFIED when supported */
-            cpl_image_delete(image);
-            image = cpl_image_load(filename, CPL_TYPE_INT, 0, 0);
-            skip_if(image == NULL);
+            con_plist = cpl_propertylist_load_regexp(filename, 0, " ESO QC | ESO PRO ",
+                                                     0);
+            skip_if(con_plist == NULL);
 
         } else {
 
@@ -395,41 +393,32 @@ cpl_error_code visir_util_join_one(cpl_frameset * framelist,
 
                 if (j == i * nz) {
 
-                    cpl_image_delete(image);
-                    image = cpl_image_load(filename, CPL_TYPE_INT, 0, 0);
-                    skip_if(image == NULL);
+                    con = cpl_image_load(filename, CPL_TYPE_INT, 0, 0);
+                    skip_if(con == NULL);
 
                 } else {
                     csum = cpl_image_load(filename, CPL_TYPE_INT, 0, 0);
                     skip_if(csum == NULL);
-                    skip_if(cpl_image_add(image, csum));
+                    skip_if(cpl_image_add(con, csum));
                 }
             }
 
-            cpl_propertylist_empty(plist);
         }
 
-        bug_if(cpl_propertylist_append_string(plist, "EXTNAME",
-                                              "CONTRIBUTION MAP"));
-
-        /* FIXME: Change to CPL_TYPE_UNSPECIFIED when supported */
-        skip_if(cpl_image_save(image, proname, CPL_BPP_32_SIGNED,
-                               plist, CPL_IO_EXTEND));
-
-        if (bpmframes == NULL) {
-
-            bug_if(cpl_propertylist_update_string(plist, "EXTNAME",
-                                                  "BAD PIXEL MAP"));
+        if (bpm == NULL) {
+            bpm = cpl_image_duplicate(con);
 
             /* FIXME: Find a better way to create a image BPM */
-            bug_if(cpl_image_threshold(image, -0.5, 1.0, 0.5, 1.0));
-            bug_if(cpl_image_multiply_scalar(image, -1.0));
-            bug_if(cpl_image_add_scalar(image, 1.0));
-            skip_if(cpl_image_save(image, proname, CPL_BPP_8_UNSIGNED,
-                                   plist, CPL_IO_EXTEND));
+            bug_if(cpl_image_threshold(bpm, -0.5, 1.0, 0.5, 1.0));
+            bug_if(cpl_image_multiply_scalar(bpm, -1.0));
+            bug_if(cpl_image_add_scalar(bpm, 1.0));
+
+            bpm_plist = cpl_propertylist_new();
+            cpl_propertylist_append_string(bpm_plist, "EXTNAME", VISIR_EXTN_BPM);
         }
     }
 
+
     if (wgtframes != NULL) {
         /* 5) The weight frame */
 
@@ -437,44 +426,106 @@ cpl_error_code visir_util_join_one(cpl_frameset * framelist,
 
         filename = cpl_frame_get_filename(frame);
 
-        cpl_propertylist_delete(plist);
-        plist = cpl_propertylist_load_regexp(filename, 0, " ESO QC | ESO PRO ",
-                                             0);
-        skip_if(plist == NULL);
+        wgt_plist = cpl_propertylist_load_regexp(filename, 0, " ESO QC | ESO PRO ",
+                                                 0);
+        skip_if(wgt_plist == NULL);
 
-        bug_if(cpl_propertylist_append_string(plist, "EXTNAME", "WEIGHT MAP"));
+        bug_if(cpl_propertylist_append_string(wgt_plist, "EXTNAME", VISIR_EXTN_WEIGHT));
 
-        /* FIXME: Change to CPL_TYPE_UNSPECIFIED when supported */
-        cpl_image_delete(image);
-        image = cpl_image_load(filename, CPL_TYPE_FLOAT, 0, 0);
-        skip_if(image == NULL);
+        wgt = cpl_image_load(filename, CPL_TYPE_FLOAT, 0, 0);
+        skip_if(wgt == NULL);
+
+        if (err == NULL) {
+            err = cpl_image_new(cpl_image_get_size_x(wgt),
+                                cpl_image_get_size_y(wgt), CPL_TYPE_FLOAT);
+            cpl_image_add_scalar(err, 1.0);
+            cpl_image_divide(err, wgt);
+            cpl_image_power(err, 0.5);
+            cpl_image_fill_rejected(err, INFINITY);
+            err_plist = cpl_propertylist_new();
+            cpl_propertylist_append_string(err_plist, "EXTNAME", VISIR_EXTN_ERROR);
+            if (bunit) {
+                cpl_propertylist_append_string(err_plist, "BUNIT", bunit);
+            }
+            skip_if(0);
+        }
 
-        /* FIXME: Change to CPL_TYPE_UNSPECIFIED when supported */
-        skip_if(cpl_image_save(image, proname, CPL_BPP_IEEE_FLOAT,
-                               plist, CPL_IO_EXTEND));
     }
 
-    for (frame = cpl_frameset_get_first_const(products);
-         frame != NULL;
-         frame = cpl_frameset_get_next_const(products)) {
-        cpl_frame * copy = cpl_frame_duplicate(frame);
-        cpl_error_code error;
+    if (err != NULL && con == NULL && bpm == NULL) {
 
-#ifdef _OPENMP
-#pragma omp critical(visir_util_join_one)
-#endif
-        error = cpl_frameset_insert(framelist, copy);
+        /* FIXME: Find a better way to create a image BPM */
+        cpl_mask * mbpm = cpl_mask_threshold_image_create(err, -DBL_EPSILON, 1e8);
+        cpl_mask_not(mbpm);
+        bpm = cpl_image_new_from_mask(mbpm);
+        cpl_mask_delete(mbpm);
+
+        bpm_plist = cpl_propertylist_new();
+        cpl_propertylist_append_string(bpm_plist, "EXTNAME", VISIR_EXTN_BPM);
+    }
+    skip_if(0);
+
+    skip_if(irplib_dfs_save_image(products, parlist, usedframes,
+                                  img, CPL_TYPE_UNSPECIFIED,
+                                  RECIPE_STRING,
+                                  procatg, img_plist,
+                                  NULL, visir_pipe_id, proname));
+
+    if (bpm) {
+        if (bpmframes) {
+            frame = irplib_framelist_get_const(bpmframes, bshared ? 0 : i);
+            bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate(frame)));
+        }
+        skip_if(cpl_image_save(bpm, proname, CPL_BPP_8_UNSIGNED,
+                               bpm_plist, CPL_IO_EXTEND));
+    }
+    if (err) {
+        if (errframes) {
+            frame = irplib_framelist_get_const(errframes, eshared ? 0 : i);
+            bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate(frame)));
+        }
+        skip_if(cpl_image_save(err, proname, CPL_TYPE_UNSPECIFIED,
+                               err_plist, CPL_IO_EXTEND));
+    }
+    if (con) {
+        if (conframes) {
+            frame = irplib_framelist_get_const(conframes, cshared ? 0 : i);
+            bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate(frame)));
+        }
+        skip_if(cpl_image_save(con, proname, CPL_TYPE_UNSPECIFIED,
+                               con_plist, CPL_IO_EXTEND));
+    }
+    if (wgt) {
+        if (wgtframes) {
+            frame = irplib_framelist_get_const(wgtframes, wshared ? 0 : i);
+            bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate(frame)));
+        }
+        skip_if(cpl_image_save(wgt, proname, CPL_TYPE_UNSPECIFIED,
+                               wgt_plist, CPL_IO_EXTEND));
+    }
+
+    FOR_EACH_FRAMESET_C(frm, products) {
+        cpl_frame * copy = cpl_frame_duplicate(frm);
+        cpl_error_code error = cpl_frameset_insert(framelist, copy);
 
         if (error) break;
     }
-    bug_if(frame != NULL);
 
     end_skip;
 
-    cpl_image_delete(image);
-    cpl_mask_delete(bpm);
+    cpl_image_delete(img);
+    cpl_image_delete(err);
+    cpl_image_delete(wgt);
+    cpl_image_delete(con);
+    cpl_image_delete(bpm);
+    cpl_image_delete(csum);
     cpl_free(proname);
-    cpl_propertylist_delete(plist);
+    cpl_propertylist_delete(img_plist);
+    cpl_propertylist_delete(err_plist);
+    cpl_propertylist_delete(wgt_plist);
+    cpl_propertylist_delete(con_plist);
+    cpl_propertylist_delete(bpm_plist);
+    cpl_propertylist_delete(qc_plist);
     cpl_frameset_delete(usedframes);
     cpl_frameset_delete(products);
 
