@@ -1,4 +1,4 @@
-/* $Id: visir_pfits.c,v 1.46 2012/02/02 10:22:05 jtaylor Exp $
+/* $Id: visir_pfits.c,v 1.53 2013-09-24 10:46:00 jtaylor Exp $
  *
  * This file is part of the VISIR Pipeline
  * Copyright (C) 2002,2003 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: jtaylor $
- * $Date: 2012/02/02 10:22:05 $
- * $Revision: 1.46 $
- * $Name: visir-3_5_1 $
+ * $Date: 2013-09-24 10:46:00 $
+ * $Revision: 1.53 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -143,7 +143,64 @@ const char * visir_pfits_get_chopnod_dir(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_chop_throw(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double(VISIR_PFITS_DOUBLE_CHOP_THROW);
+    if (cpl_propertylist_has(self, VISIR_PFITS_DOUBLE_CHOP_THROW)) {
+        return visir_pfits_get_double(VISIR_PFITS_DOUBLE_CHOP_THROW);
+    }
+    else {
+        return visir_pfits_get_double("ESO DET CHOP THROW");
+    }
+}
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The chopping frequency
+  @param    self       property list to read from
+  @return   chopping frequency
+ */
+/*----------------------------------------------------------------------------*/
+double visir_pfits_get_chop_freq(const cpl_propertylist * self)
+{
+    if (cpl_propertylist_has(self, VISIR_PFITS_DOUBLE_CHOP_FREQ)) {
+        return visir_pfits_get_double(VISIR_PFITS_DOUBLE_CHOP_FREQ);
+    }
+    else {
+        return visir_pfits_get_double("ESO DET CHOP FREQ");
+    }
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The chopping status
+  @param    self       property list to read from
+  @return   chopping status
+ */
+/*----------------------------------------------------------------------------*/
+double visir_pfits_get_chop_stat(const cpl_propertylist * self)
+{
+    if (cpl_propertylist_has(self, VISIR_PFITS_BOOL_CHOP_STATUS)) {
+        return cpl_propertylist_get_bool(self, VISIR_PFITS_BOOL_CHOP_STATUS);
+    }
+    else {
+        return cpl_propertylist_get_bool(self, "ESO DET CHOP ST");
+    }
+}
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The chopping position angle in rad
+  @param    self       property list to read from
+  @return   Chopping position angle in rad
+ */
+/*----------------------------------------------------------------------------*/
+double visir_pfits_get_chop_posang(const cpl_propertylist * self)
+{
+    if (!cpl_propertylist_has(self, VISIR_PFITS_DOUBLE_CHOP_POSANG)) {
+        return 0.;
+    }
+    return CPL_MATH_RAD_DEG *
+        cpl_propertylist_get_double(self, VISIR_PFITS_DOUBLE_CHOP_POSANG);
 }
 
 
@@ -157,8 +214,7 @@ double visir_pfits_get_chop_throw(const cpl_propertylist * self)
 double visir_pfits_get_chop_pthrow(const cpl_propertylist * self)
 {
     const double throw = visir_pfits_get_chop_throw(self);
-    const char * sscale = visir_pfits_get_pixscale(self);
-    const double pscale       = sscale ? atof(sscale) : 0.0;
+    const double pscale = visir_pfits_get_pixscale(self);
     return pscale > 0.0 ? throw / pscale : 0.0;
 }
 
@@ -244,7 +300,23 @@ double visir_pfits_get_dec(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_dit(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO DET DIT");
+    if (cpl_propertylist_has(self, VISIR_PFITS_DOUBLE_SEQ1_DIT))
+        return visir_pfits_get_double(VISIR_PFITS_DOUBLE_SEQ1_DIT);
+    return visir_pfits_get_double(VISIR_PFITS_DOUBLE_DIT);
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The NAVRG
+  @param    self       property list to read from
+  @return   the requested value
+ */
+/*----------------------------------------------------------------------------*/
+int visir_pfits_get_navrg(const cpl_propertylist * self)
+{
+    if (cpl_propertylist_has(self, VISIR_PFITS_INT_NAVRG))
+        return visir_pfits_get_int(VISIR_PFITS_INT_NAVRG);
+    return 1;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -408,6 +480,35 @@ int visir_pfits_get_ndit(const cpl_propertylist * self)
 {   
     return visir_pfits_get_int("ESO DET NDIT");
 }
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The NAXIS1 key
+  @param    self       property list to read from
+  @return   the requested value
+ */
+/*----------------------------------------------------------------------------*/
+int visir_pfits_get_naxis1(const cpl_propertylist * self)
+{
+    if (cpl_propertylist_has(self, "ZNAXIS1"))
+        return visir_pfits_get_int("ZNAXIS1");
+    else
+        return visir_pfits_get_int("NAXIS1");
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The NAXIS2 key
+  @param    self       property list to read from
+  @return   the requested value
+ */
+/*----------------------------------------------------------------------------*/
+int visir_pfits_get_naxis2(const cpl_propertylist * self)
+{
+    if (cpl_propertylist_has(self, "ZNAXIS2"))
+        return visir_pfits_get_int("ZNAXIS2");
+    else
+        return visir_pfits_get_int("NAXIS2");
+}
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -418,7 +519,70 @@ int visir_pfits_get_ndit(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 int visir_pfits_get_naxis3(const cpl_propertylist * self)
 {
-    return visir_pfits_get_int("NAXIS3");
+    if (cpl_propertylist_has(self, "ZNAXIS3"))
+        return visir_pfits_get_int("ZNAXIS3");
+    else
+        return visir_pfits_get_int("NAXIS3");
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The WIN NX key
+  @param    self       property list to read from
+  @return   the requested value or -1 if not present
+ */
+/*----------------------------------------------------------------------------*/
+int visir_pfits_get_win_nx(const cpl_propertylist * self)
+{
+    if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN NX")) {;
+        return visir_pfits_get_int("ESO DET ACQ1 WIN NX");
+    }
+    return -1;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The WIN NY key
+  @param    self       property list to read from
+  @return   the requested value or -1 if not present
+ */
+/*----------------------------------------------------------------------------*/
+int visir_pfits_get_win_ny(const cpl_propertylist * self)
+{
+    if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN NY")) {;
+        return visir_pfits_get_int("ESO DET ACQ1 WIN NY");
+    }
+    return -1;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The WIN STRX key
+  @param    self       property list to read from
+  @return   the requested value or -1 if not present
+ */
+/*----------------------------------------------------------------------------*/
+int visir_pfits_get_start_x(const cpl_propertylist * self)
+{
+    if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN STRX")) {;
+        return visir_pfits_get_int("ESO DET ACQ1 WIN STRX");
+    }
+    return -1;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The WIN STRY key
+  @param    self       property list to read from
+  @return   the requested value or -1 if not present
+ */
+/*----------------------------------------------------------------------------*/
+int visir_pfits_get_start_y(const cpl_propertylist * self)
+{
+    if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN STRY")) {;
+        return visir_pfits_get_int("ESO DET ACQ1 WIN STRY");
+    }
+    return -1;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -461,12 +625,16 @@ const char * visir_pfits_get_nodpos(const cpl_propertylist * self)
 /**
   @brief    The pixel scale
   @param    self       property list to read from
-  @return   pointer to statically allocated character string
+  @return   on success double pixel scale, -1. on failure
  */
 /*----------------------------------------------------------------------------*/
-const char *  visir_pfits_get_pixscale(const cpl_propertylist * self)
+double visir_pfits_get_pixscale(const cpl_propertylist * self)
 {
-    return visir_pfits_get_string("ESO INS PFOV");
+    const char * sscale = visir_pfits_get_string(VISIR_PFITS_STRING_PIXSCALE);
+    if (sscale && strlen(sscale) > 0)
+        return atof(sscale);
+    else
+        return -1.;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -640,7 +808,49 @@ double visir_pfits_get_volt2dctb9(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_wlen(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO INS GRAT1 WLEN");
+    if (cpl_propertylist_has(self, VISIR_PFITS_DOUBLE_PWLEN)) {
+        return visir_pfits_get_double(VISIR_PFITS_DOUBLE_PWLEN);
+    }
+    return visir_pfits_get_double(VISIR_PFITS_DOUBLE_WLEN);
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The relative weight of the image compared to the other images
+  @param    self       property list to read from
+  @return   the requested value
+
+  e.g. center image in parallel mode has double weight
+
+ */
+/*----------------------------------------------------------------------------*/
+double visir_pfits_get_img_weight(const cpl_propertylist * self)
+{
+    double imgwgt = 1.;
+    if (cpl_propertylist_has(self, "ESO DRS IMGWGT"))
+        imgwgt = cpl_propertylist_get_float(self, "ESO DRS IMGWGT");
+
+    return imgwgt;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The number of input beams
+  @param    self       property list to read from
+  @return   the requested value
+
+  with chop-nodding 4, only nodding 2
+
+ */
+/*----------------------------------------------------------------------------*/
+int visir_pfits_get_nbeams(const cpl_propertylist * self)
+{
+    const cpl_boolean chop_on =
+        cpl_propertylist_get_bool(self, VISIR_PFITS_BOOL_CHOP_STATUS);
+
+    /* FIXME: starring mode support? */
+
+    return chop_on ? 4 : 2;
 }
 
 /**@}*/
