@@ -1,4 +1,4 @@
-/* $Id: visir_spc_optmod.c,v 1.24 2012/01/31 08:16:58 llundin Exp $
+/* $Id: visir_spc_optmod.c,v 1.25 2012-08-03 01:55:22 llundin Exp $
  *
  * This file is part of the VISIR Pipeline
  * Copyright (C) 2002,2003 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/01/31 08:16:58 $
- * $Revision: 1.24 $
- * $Name: visir-3_5_1 $
+ * $Date: 2012-08-03 01:55:22 $
+ * $Revision: 1.25 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -67,6 +67,7 @@ enum visir_spc_mode_ {
     VISIR_SPC_M_LLWN,
     VISIR_SPC_M_LSWQ,
     VISIR_SPC_M_LLWQ,
+    VISIR_SPC_M_LRP,
     VISIR_SPC_M_MSWN,
     VISIR_SPC_M_MLWN,
     VISIR_SPC_M_MSWQ,
@@ -170,9 +171,10 @@ static void visir_spc_optmod_scan_angle(visir_optmod_private *);
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Initialize VISIR spectroscopy instrument specific constants
-  @param    resol The resolution ( VISIR_SPC_R_[L|M|H|GH]R )
-  @param    wlen  The central wavelength [m]
+  @param    resol The resolution ( VISIR_SPC_R_[[L|M|H|GH]R|RP] )
+  @param    wlen  The central wavelength [m] (FIXME: ignored for _LRP ?)
   @param    pins  Pointer to instrument settings structure
+  @param    is_aqu whether to build model for the aquarius detector
   @return   0 iff successful.
 
   The possible error conditions are:
@@ -182,7 +184,7 @@ static void visir_spc_optmod_scan_angle(visir_optmod_private *);
  */
 /*----------------------------------------------------------------------------*/
 int visir_spc_optmod_init(visir_spc_resol resol, double wlen,
-                          visir_optmod * pins)
+                          visir_optmod * pins, int is_aqu)
 {
 
     visir_optmod_private * self = (visir_optmod_private*)pins;
@@ -202,6 +204,13 @@ int visir_spc_optmod_init(visir_spc_resol resol, double wlen,
        Constants here are coded in micron - convert to m */
     wlen *= 1e6;
     switch (resol) {
+        case VISIR_SPC_R_LRP:
+          {
+              mode = VISIR_SPC_M_LRP;
+              wlen = VISIR_SPC_LRP_CWLEN;
+              break;
+          }
+
         case VISIR_SPC_R_LR:
           {
                  if ( 7.5 <= wlen && wlen <= 10.2) mode = VISIR_SPC_M_LSWN;
@@ -284,6 +293,10 @@ int visir_spc_optmod_init(visir_spc_resol resol, double wlen,
 
     /* Switch on the resolution */
     switch (resol) {
+    case VISIR_SPC_R_LRP:
+        {
+            break;
+        }
     case VISIR_SPC_R_LR:
         {
             self->angle_a  = 6.708;
@@ -689,6 +702,7 @@ int visir_spc_optmod_init(visir_spc_resol resol, double wlen,
                 self->angle_bm = 64.8519;
                 self->angle_b0 = 64.5393;
                 self->angle_b1 = 65.1641;
+
             } else {
                 /* side 'B' */
                 self->angle_a  = 64.8701;
@@ -787,21 +801,42 @@ int visir_spc_optmod_init(visir_spc_resol resol, double wlen,
     default:;
     }
 
-    self->angle_a  *= rad_per_deg;
-    self->angle_b0 *= rad_per_deg;
-    self->angle_bm *= rad_per_deg;
-    self->angle_b1 *= rad_per_deg;
+    if (is_aqu && (resol == VISIR_SPC_R_HR || resol == VISIR_SPC_R_GHR)) {
+        /* adapt VLT-TRE-VIS-14321-5046 measurements to new detector size and
+         * pfov */
+        self->factor *= 0.127 / 0.0757;
+        self->ld /= 0.127 / 0.0757;
+        /* drs: b0 at pixel 1, bm at 128.5, b1 at 256 with pfov 0.127
+         * aqu: bm at 512.5, b1 at 1024 with pfov 0.757
+         * => d = 128.5 / tan(b1 - bm) -> (b1 - bm)_new = atan((512.5 / 1.6777) / d) */
+        if (self->angle_b0 < self->angle_bm) { /* A side */
+            self->angle_b0 = self->angle_bm - (self->angle_bm - self->angle_b0) * 2.377;
+            self->angle_b1 = self->angle_bm + (self->angle_b1 - self->angle_bm) * 2.377;
+        }
+        else { /* B side */
+            self->angle_b0 = self->angle_bm + (self->angle_b0 - self->angle_bm) * 2.377;
+            self->angle_b1 = self->angle_bm - (self->angle_bm - self->angle_b1) * 2.377;
+        }
+        self->offset += 162;
+    }
 
-    /* Length constants are coded in micron - convert to m */
-    self->d        *= 1e-6;
-    self->dcolbeam *= 1e-6;
+    if (resol != VISIR_SPC_R_LRP) {
+        self->angle_a  *= rad_per_deg;
+        self->angle_b0 *= rad_per_deg;
+        self->angle_bm *= rad_per_deg;
+        self->angle_b1 *= rad_per_deg;
 
-    assert( self->m > 0);
+        /* Length constants are coded in micron - convert to m */
+        self->d        *= 1e-6;
+        self->dcolbeam *= 1e-6;
 
-    visir_spc_optmod_scan_angle(self);
+        assert( self->m > 0);
 
-    self->sinus_sum = sin(self->angle_a  + self->angle_scan)
-                    + sin(self->angle_bm + self->angle_scan);
+        visir_spc_optmod_scan_angle(self);
+
+        self->sinus_sum = sin(self->angle_a  + self->angle_scan)
+            + sin(self->angle_bm + self->angle_scan);
+    }
 
     return 0;
 
@@ -831,6 +866,12 @@ double visir_spc_optmod_wlen(const visir_optmod * pins, double * pwl0,
 
     if (self == NULL) return -1;
 
+    if (self->mode == VISIR_SPC_M_LRP) {
+        if (pwl0) *pwl0 = VISIR_SPC_LRP_WLEN0;
+        if (pwl1) *pwl1 = VISIR_SPC_LRP_WLEN1;
+        return VISIR_SPC_LRP_CWLEN;
+    }
+
     /* The wavelength on the center of the 1st pixel */
     if (pwl0) *pwl0 = self->d/self->m*( sin(self->angle_a  + self->angle_scan)
                                       + sin(self->angle_b0 + self->angle_scan));
@@ -1018,8 +1059,10 @@ double visir_spc_optmod_resolution(const visir_optmod * pins)
 
     if (self == NULL) return -1;
 
-    return self->dcolbeam * self->sinus_sum
-        /(2.0 * self->wlen * cos(self->angle_a  + self->angle_scan));
+    return self->mode == VISIR_SPC_M_LRP
+        ? VISIR_SPC_LRP_RESOL
+        : self->dcolbeam * self->sinus_sum
+        / (2.0 * self->wlen * cos(self->angle_a  + self->angle_scan));
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1043,8 +1086,10 @@ double visir_spc_optmod_dispersion(const visir_optmod * pins)
 
     if (self == NULL) return -1;
 
-    return self->ld * self->sinus_sum
-        /(self->wlen * cos(self->angle_bm + self->angle_scan));
+    return self->mode == VISIR_SPC_M_LRP
+        ? VISIR_SPC_LRP_RESOL_DISP
+        : self->ld * self->sinus_sum
+        / (self->wlen * cos(self->angle_bm + self->angle_scan));
 }
 
 /**@}*/
