@@ -1,4 +1,4 @@
-/* $Id: visir_spc_photom.c,v 1.54 2010/09/30 06:48:35 llundin Exp $
+/* $Id: visir_spc_photom.c,v 1.64 2013-03-04 09:00:00 jtaylor Exp $
  *
  * This file is part of the VISIR Pipeline
  * Copyright (C) 2002,2003 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/09/30 06:48:35 $
- * $Revision: 1.54 $
- * $Name: visir-3_5_1 $
+ * $Author: jtaylor $
+ * $Date: 2013-03-04 09:00:00 $
+ * $Revision: 1.64 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -54,13 +54,13 @@ static cpl_bivector * visir_spc_phot_model_from_cat(const char *, double,
                                                     double);
 static int visir_spc_phot_model_rebin(const cpl_bivector *, cpl_table *);
 
-static cpl_error_code visir_spc_sens_stat(const visir_spc_phot_config *,
-                                          double *, double *,
+static cpl_error_code visir_spc_sens_stat(const visir_spc_config *,
+                                          double *, double *, double *,
                                           const cpl_table *);
 
 static cpl_error_code visir_spc_phot_qc(cpl_propertylist *,
-                                        const char *,
-                                        double, double, double);
+                                        const char *, double, double, double,
+                                        double, double);
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -70,90 +70,56 @@ static cpl_error_code visir_spc_phot_qc(cpl_propertylist *,
 
 /**@{*/
 
+
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    Compute the sensitivity
-  @param    combined       The combined image 
+  @brief    Compute the sensitivity from an extracted spectrum
   @param    rawframes      List of input raw frames
-  @param    recipename     The name of the caller recipe
-  @param    parlist        The recipe parameter list
   @param    pconfig        The config struct
+  @param    plist          The properties (DIT, NDIT etc.)
   @param    star_cat       The standard star catalog
-  @param    spc_cal_lines  The sky lines table
-  @param    spc_cal_qeff   The quantum efficiency
   @param    pweight2d      2D weights image to be created or NULL on error
   @param    qclist         QC Properties are appended to this list
-  @param    do_ech         True iff the image is of type echelle
-  @param    wlen           Central wavelength
-  @param    slitw          The slit width
-  @param    temp           The telescope (M1) temperature [Kelvin]
-  @param    fwhm           The spectral FWHM [pixel]
-  @param    resol          Resolution
-  @return   a table or NULL in error case
+  @param    spc_table      The Spectral Table
+  @param    dit_key        The key of the DIT card
+  @return   a table or NULL on error
+  @see visir_spc_phot_sensit2()
  */
 /*----------------------------------------------------------------------------*/
-cpl_table * visir_spc_phot_sensit(
-        cpl_image    ** combined,
-        const irplib_framelist * rawframes,
-        const char   * recipename,
-        const cpl_parameterlist * parlist,
-        const visir_spc_phot_config * pconfig,
-        const char   * star_cat,
-        const char   * spc_cal_lines,
-        const char   * spc_cal_qeff,
-        cpl_image   ** pweight2d,
-        cpl_propertylist * qclist,
-        cpl_boolean    do_ech,
-        double         wlen,
-        double         slitw,
-        double         temp,
-        double         fwhm,
-        const visir_spc_resol resol)
+cpl_error_code
+visir_spc_phot_sensit(const irplib_framelist * rawframes,
+                      const visir_spc_config * pconfig,
+                      const cpl_propertylist * plist,
+                      const char   * star_cat,
+                      cpl_image   ** pweight2d,
+                      cpl_propertylist * qclist,
+                      cpl_table *   spc_table,
+                      const char * dit_key)
+
 {
     cpl_errorstate cleanstate = cpl_errorstate_get();
-    const cpl_propertylist * plist;
-    double                  exptime;
-    double                  dit, ndit;
-    int                     ncycles, nnod;
-    int                     icol1, icol2;
-    int                     jcol1, jcol2;
-    double                  ra, dec;
-    cpl_imagelist       *   hcycle = NULL;
-    cpl_image           *   imhcycle = NULL;
-    cpl_image           *   comorder = NULL;
-    cpl_image           *   order = NULL;
-    const char          *   star_name;
-    cpl_bivector        *   flux_model = NULL;
-    cpl_table           *   spc_table = NULL;
-    double                  factor;
-    double              *   ext_array;
-    double              *   spc_array;
-    double              *   err_array;
-    double                  sens_mean = -1.0; /* Avoid (false) uninit warning */
-    double                  sens_stdev = -1.0; /* Ditto */
-    int                     i;
-    
-
-    *pweight2d = NULL;
+    cpl_bivector * flux_model = NULL;
+    double * ext_array;
+    double * spc_array;
+    double * err_array;
+    double sens_median = -1.0; /* Avoid (false) uninit warning */
+    double sens_mean = -1.0; /* Avoid (false) uninit warning */
+    double sens_stdev = -1.0; /* Ditto */
+    double exptime, factor, exectime = -1.;
+    /* Get RA / DEC */
+    const double ra  = visir_pfits_get_ra(plist);
+    const double dec = visir_pfits_get_dec(plist);
+    const char * star_name;
 
     skip_if (0);
 
-    skip_if (rawframes == NULL);
-
-    jcol1 = visir_parameterlist_get_int(parlist, recipename,
-                                        VISIR_PARAM_REJLEFT);
-    jcol2 = visir_parameterlist_get_int(parlist, recipename,
-                                        VISIR_PARAM_REJRIGHT);
-
-    skip_if (0);
-
-    /* DIT must be present every where */  
-    skip_if(irplib_framelist_contains(rawframes, VISIR_PFITS_DOUBLE_DIT,
+    /* DIT must be present every where */
+    skip_if(irplib_framelist_contains(rawframes, dit_key,
                                       CPL_TYPE_DOUBLE, CPL_FALSE, 0.0));
-    if(irplib_framelist_contains(rawframes, VISIR_PFITS_DOUBLE_DIT,
+    if(irplib_framelist_contains(rawframes, dit_key,
                                  CPL_TYPE_DOUBLE, CPL_TRUE, 1e-5)) {
         /* Allow 0.1 ms difference - or warn */
-        /* FIXME: The recipe does not properly handle non-uniform DITSs */  
+        /* FIXME: The recipe does not properly handle non-uniform DITSs */
         visir_error_reset("DIT differs by more than %g", 1e-5);
     }
 
@@ -176,120 +142,23 @@ cpl_table * visir_spc_phot_sensit(
         visir_error_reset("Rawframe(s) missing standard star name");
     }
 
+    if (cpl_propertylist_has(plist, "ESO QC EXPTIME"))
+        exptime = cpl_propertylist_get_double(plist, "ESO QC EXPTIME");
+    else {
+        const int nnod = irplib_framelist_get_size(rawframes);
+        exptime = visir_utils_get_exptime(nnod, plist);
+    }
+    cpl_msg_info(cpl_func, "Exposure time: %.3f", exptime);
+    if (cpl_propertylist_has(plist, "ESO QC EXECTIME"))
+        exectime = cpl_propertylist_get_double(plist, "ESO QC EXECTIME");
 
-    /* Get the first frame */
-    plist = irplib_framelist_get_propertylist_const(rawframes, 0);
+    skip_if (exptime <= 0.0);
 
-    skip_if (0);
-
-    /* Get the total exposure time */
-    /* DIT */
-    dit = visir_pfits_get_dit(plist);
-    /* NDIT */
-    ndit = visir_pfits_get_ndit(plist);
-    /* NNOD */
-    nnod = irplib_framelist_get_size(rawframes);
-    /* Number of chopping cycles */
-    ncycles = visir_pfits_get_chop_ncycles(plist);
-    /* exptime *= 2 because of chopping */
-    exptime = 2 * dit * ndit * nnod * ncycles;
-
-    skip_if (0);
+    factor = sqrt(exptime)/6.0; /* sqrt(exptime/3600) * 10; */
 
     /* Get the standard star name */
     star_name = visir_pfits_get_starname(plist);
     if (star_name == NULL) visir_error_reset("Could not get standard star name");
-    
-    /* Get RA / DEC */
-    ra  = visir_pfits_get_ra(plist);
-    dec = visir_pfits_get_dec(plist);
-    
-    skip_if (0);
-
-    /* Get the HCYCLE frame */
-    hcycle = visir_load_imagelist(rawframes, 0, CPL_FALSE);
-    skip_if (0);
-
-    skip_if (pconfig->do_fixcombi && visir_spc_det_fix(combined, 1, CPL_TRUE,
-                               wlen, resol,
-                               pconfig->phi,
-                               pconfig->ksi,
-                               pconfig->eps,
-                               pconfig->delta,
-                               pconfig->plot));
-
-
-    imhcycle = cpl_imagelist_get(hcycle, 0);    
-    skip_if (0);
-   
-    skip_if (visir_spc_det_fix(&imhcycle, 1, CPL_FALSE,
-                               wlen, resol,
-                               pconfig->phi,
-                               pconfig->ksi,
-                               pconfig->eps,
-                               pconfig->delta,
-                               pconfig->plot));
-
-    if (do_ech) {
-        skip_if (visir_spc_echelle_limit(&icol1, &icol2, wlen,
-                                         pconfig->orderoffset,
-                                         1, cpl_image_get_size_y(*combined)));
-        skip_if (visir_qc_append_background(qclist, rawframes, icol1, icol2));
-
-    } else {
-        icol1 = 1;
-        icol2 = cpl_image_get_size_x(imhcycle);
-
-        skip_if (visir_qc_append_background(qclist, rawframes, 0, 0));
-    }
-
-    if (do_ech) {
-        if (jcol1 != 0) {
-            cpl_msg_info(cpl_func, "Ignoring %d leftmost columns from %d to %d",
-                         jcol1, icol1, icol1 + jcol1);
-            icol1 += jcol1;
-        }
-        if (jcol2 != 0) {
-            cpl_msg_info(cpl_func, "Ignoring %d rightmost columns from %d to %d",
-                         jcol2, icol2 - jcol2, icol2);
-            icol2 -= jcol2;
-        }
-    } else {
-        if (jcol1 != 0) {
-            cpl_msg_info(cpl_func, "Ignoring %d leftmost columns", jcol1);
-            icol1 += jcol1;
-        }
-        if (jcol2 != 0) {
-            cpl_msg_info(cpl_func, "Ignoring %d rightmost columns", jcol2);
-            icol2 -= jcol2;
-        }
-    }
-
-    if (icol1 != 1 || icol2 != cpl_image_get_size_x(imhcycle)) {
-        order = visir_spc_column_extract(imhcycle, icol1, icol2,
-                                          pconfig->plot);
-        skip_if (0);
-
-        comorder = visir_spc_column_extract(*combined, icol1, icol2,
-                                             pconfig->plot);
-        skip_if (0);
-
-    } else {
-        order = imhcycle;
-        comorder = *combined;
-    }
-
-    /* Extract the spectrum */
-    skip_if (visir_spc_extract_wcal(comorder,
-                                    order, wlen, slitw, temp, fwhm,
-                                    resol, pconfig->orderoffset,
-                                    spc_cal_lines, spc_cal_qeff,
-                                    &spc_table, pweight2d,
-                                    qclist,
-                                    pconfig->plot));
-      
-    cpl_imagelist_delete(hcycle);
-    hcycle = NULL;
 
     /* Get the flux model from the catalog */
     if ((flux_model=visir_spc_phot_model_from_cat(star_cat, ra, dec)) == NULL) {
@@ -306,43 +175,42 @@ cpl_table * visir_spc_phot_sensit(
     flux_model = NULL;
 
     /* Compute the sensitivity */
-    /* Sensitivity = model * error * 10 * sqrt(DIT*NDIT*NFILES*NCHOP*2) 
-        / extracted */
-    cpl_table_duplicate_column(spc_table, "SENSITIVITY", spc_table, 
-            "STD_STAR_MODEL");
+    /* Sensitivity = model * error * 10 * sqrt(DIT*NDIT*NFILES*NCHOP*2)
+       / extracted */
+    cpl_table_duplicate_column(spc_table, "SENSITIVITY", spc_table,
+                               "STD_STAR_MODEL");
     cpl_table_set_column_unit(spc_table, "SENSITIVITY", "mJy");
 
     ext_array = cpl_table_get_data_double(spc_table, "SENSITIVITY");
     spc_array = cpl_table_get_data_double(spc_table, "SPC_EXTRACTED");
     err_array = cpl_table_get_data_double(spc_table, "SPC_ERROR");
 
-    factor = sqrt(exptime)/6.0; /* sqrt(exptime/3600) * 10; */
-    for (i = 0; i < cpl_table_get_nrow(spc_table); i++) {
+    for (cpl_size i = 0; i < cpl_table_get_nrow(spc_table); i++) {
         ext_array[i] *= factor * err_array[i];
         if (ext_array[i] < 0 || spc_array[i] <= 0) {
             cpl_msg_warning(cpl_func, "Setting non-physical sensitivity in row %d "
-                            "to 0: %g/%g", i+1, ext_array[i], spc_array[i]);
+                            "to 0: %g/%g", (int)i+1, ext_array[i], spc_array[i]);
             ext_array[i] = 0;
         } else {
             ext_array[i] /= spc_array[i];
         }
     }
 
-    skip_if(visir_spc_sens_stat(pconfig, &sens_mean, &sens_stdev, spc_table));
+    skip_if(visir_spc_sens_stat(pconfig, &sens_median, &sens_mean, &sens_stdev, spc_table));
 
     /* This column is not part of the product */
     skip_if (cpl_table_erase_column(spc_table, "SPC_EMISSIVITY"));
 
-    skip_if (visir_spc_phot_qc(qclist, star_name, exptime, sens_mean,
-                               sens_stdev));
+    skip_if (visir_spc_phot_qc(qclist, star_name, exptime, exectime,
+                               sens_median, sens_mean, sens_stdev));
 
     /* Plot the results */
     if (pconfig->plot) {
         visir_table_plot("", "t 'Extracted spectrum' w lines", "", spc_table,
                          "WLEN", "SPC_EXTRACTED");
-        visir_table_plot("", "t 'Extracted spectrum error' w lines", "", 
+        visir_table_plot("", "t 'Extracted spectrum error' w lines", "",
                          spc_table, "WLEN", "SPC_ERROR");
-        visir_table_plot("", "t 'Standard star model' w lines", "", spc_table, 
+        visir_table_plot("", "t 'Standard star model' w lines", "", spc_table,
                          "WLEN", "STD_STAR_MODEL");
         visir_table_plot("set grid;", "t 'Sensitivity (mJy)' w lines", "",
                          spc_table, "WLEN", "SENSITIVITY");
@@ -350,12 +218,150 @@ cpl_table * visir_spc_phot_sensit(
     }
 
     end_skip;
-
-    cpl_imagelist_delete(hcycle);
     cpl_bivector_delete(flux_model);
 
-    if (order != imhcycle) cpl_image_delete(order);
-    if (comorder != *combined) cpl_image_delete(comorder);
+    return cpl_error_get_code();
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Extract spectrum from image and compute sensitivity
+  @param    combined       The combined image
+  @param    rawframes      List of input raw frames
+  @param    pconfig        The config struct
+  @param    star_cat       The standard star catalog
+  @param    spc_cal_lines  The sky lines table
+  @param    spc_cal_qeff   The quantum efficiency
+  @param    pweight2d      2D weights image to be created or NULL on error
+  @param    qclist         QC Properties are appended to this list
+  @param    do_ech         True iff the image is of type echelle
+  @param    wlen           Central wavelength
+  @param    slitw          The slit width
+  @param    temp           The telescope (M1) temperature [Kelvin]
+  @param    fwhm           The spectral FWHM [pixel]
+  @param    resol          Resolution
+  @param    dit_key        The key of the DIT card
+  @return   a table or NULL on error
+  @see visir_spc_phot_sensit()
+ */
+/*----------------------------------------------------------------------------*/
+cpl_table *
+visir_spc_phot_sensit_from_image(cpl_image    ** combined,
+                                 const irplib_framelist * rawframes,
+                                 const visir_spc_config * pconfig,
+                                 const char   * star_cat,
+                                 const char   * spc_cal_lines,
+                                 const char   * spc_cal_qeff,
+                                 cpl_image   ** pweight2d,
+                                 cpl_propertylist * qclist,
+                                 cpl_boolean    do_ech,
+                                 double         wlen,
+                                 double         slitw,
+                                 double         temp,
+                                 double         fwhm,
+                                 const visir_spc_resol resol,
+                                 const char * dit_key)
+{
+    const cpl_propertylist * plist;
+    const cpl_frame     *   frm = NULL;
+    cpl_imagelist       *   hcycle = NULL;
+    cpl_image           *   imhcycle = NULL;
+    cpl_image           *   flipped = NULL;
+    cpl_image           *   comorder = NULL;
+    cpl_image           *   order = NULL;
+    cpl_table           *   spc_table = NULL;
+    visir_data_type         dtype;
+
+    *pweight2d = NULL;
+
+    skip_if (rawframes == NULL);
+
+    /* Get the first frame */
+    plist = irplib_framelist_get_propertylist_const(rawframes, 0);
+    frm = irplib_framelist_get_const(rawframes, 0);
+
+    skip_if (visir_get_data_type(frm, plist, &dtype, NULL));
+
+    /* Get the HCYCLE frame */
+    hcycle = visir_load_imagelist(rawframes, 0, CPL_FALSE);
+    skip_if (0);
+
+    skip_if (pconfig->do_fixcombi &&
+             visir_spc_det_fix(combined, 1, CPL_TRUE,
+                               wlen, resol,
+                               pconfig->phi,
+                               pconfig->ksi,
+                               pconfig->eps,
+                               pconfig->delta,
+                               pconfig->plot));
+
+
+    imhcycle = cpl_imagelist_unset(hcycle, 0);
+    skip_if (0);
+
+    skip_if (visir_spc_det_fix(&imhcycle, 1, CPL_FALSE,
+                               wlen, resol,
+                               pconfig->phi,
+                               pconfig->ksi,
+                               pconfig->eps,
+                               pconfig->delta,
+                               pconfig->plot));
+
+    /* Convert the combined image */
+    flipped = visir_spc_flip(*combined, wlen, resol, dtype);
+    skip_if (0);
+    cpl_image_delete(*combined);
+    *combined = flipped;
+    flipped = NULL;
+
+    /* Convert the half cycle image */
+    flipped = visir_spc_flip(imhcycle, wlen, resol, dtype);
+    skip_if (0);
+    cpl_image_delete(imhcycle);
+    imhcycle = flipped;
+    flipped = NULL;
+
+    skip_if(visir_spc_extract_order(&order, &comorder,
+                                    *combined, imhcycle,
+                                    wlen, pconfig, do_ech,
+                                    visir_data_is_aqu(dtype)));
+
+    if (do_ech) {
+        int icol1, icol2;
+        skip_if (visir_spc_echelle_limit(&icol1, &icol2, wlen,
+                                         pconfig->orderoffset,
+                                         1, cpl_image_get_size_y(*combined),
+                                         visir_data_is_aqu(dtype)));
+        skip_if(visir_qc_append_background(qclist, rawframes, icol1, icol2));
+    }
+    else
+        skip_if(visir_qc_append_background(qclist, rawframes, 0, 0));
+
+    /* Extract the spectrum */
+    skip_if (visir_spc_extract_wcal(comorder,
+                                    order, wlen, slitw, temp, fwhm,
+                                    resol, pconfig->orderoffset,
+                                    spc_cal_lines, spc_cal_qeff,
+                                    &spc_table, pweight2d,
+                                    qclist,
+                                    pconfig->plot, CPL_FALSE,
+                                    visir_data_is_aqu(dtype)));
+
+    visir_spc_phot_sensit(rawframes, pconfig, plist, star_cat, pweight2d,
+                          qclist, spc_table, dit_key);
+
+    cpl_imagelist_delete(hcycle);
+    hcycle = NULL;
+
+
+    end_skip;
+
+    cpl_imagelist_delete(hcycle);
+
+    cpl_image_delete(flipped);
+    cpl_image_delete(imhcycle);
+    cpl_image_delete(order);
+    cpl_image_delete(comorder);
 
     if (cpl_error_get_code()) {
         cpl_table_delete(spc_table);
@@ -375,7 +381,7 @@ cpl_table * visir_spc_phot_sensit(
   @param    star_cat    the star catalog
   @param    ra          the RA value in degrees
   @param    dec         the DEC value in degrees
-  @return   The flux model in a cpl_bivector or NULL in error case
+  @return   The flux model in a cpl_bivector or NULL on error
 
   The catalog contains wavelengths in microns and model values in
   W/m2/m. To convert to mJy, they must be multiplied by
@@ -419,12 +425,16 @@ static cpl_bivector * visir_spc_phot_model_from_cat(const char * star_cat,
     /* Get the RA and DEC columns */
     dd  = cpl_table_get_data_double_const(catalog, "RA");
     skip_if (dd == NULL);
+    IRPLIB_DIAG_PRAGMA_PUSH_IGN(-Wcast-qual);
     v_ra = cpl_vector_wrap(nb_stars, (double*)dd); /* v_ra is _not_ modified */
+    IRPLIB_DIAG_PRAGMA_POP;
     bug_if( v_ra  == NULL);
 
     dd  = cpl_table_get_data_double_const(catalog, "DEC");
     skip_if (dd == NULL);
+    IRPLIB_DIAG_PRAGMA_PUSH_IGN(-Wcast-qual);
     v_dec = cpl_vector_wrap(nb_stars, (double*)dd); /* v_dec is _not_ modified */
+    IRPLIB_DIAG_PRAGMA_POP;
     bug_if( v_dec  == NULL);
 
     /* Find the star closest to the given ra, dec */
@@ -478,7 +488,9 @@ static cpl_bivector * visir_spc_phot_model_from_cat(const char * star_cat,
 
     (void)cpl_vector_unwrap(v_ra);
     (void)cpl_vector_unwrap(v_dec);
+    IRPLIB_DIAG_PRAGMA_PUSH_IGN(-Wcast-qual);
     cpl_table_delete((cpl_table*)catalog);
+    IRPLIB_DIAG_PRAGMA_POP;
 
     if (cpl_error_get_code()) {
         /* At this point model is certain to be NULL */
@@ -555,13 +567,15 @@ static int visir_spc_phot_model_rebin(
 /**
   @brief    Compute some statistics on the sensitivity
   @param    pconfig     The config struct
+  @param    psens_median The median of the sensitivity (over the wavelenghts)
   @param    psens_mean   The mean of the sensitivity (over the wavelenghts)
   @param    psens_stdev  The stdev of the sensitivity (over the wavelenghts)
   @param    spc_table   The Spectral Table
   @return   CPL_ERROR_NONE or the relevant #_cpl_error_code_
  */
 /*----------------------------------------------------------------------------*/
-static cpl_error_code visir_spc_sens_stat(const visir_spc_phot_config * pconfig,
+static cpl_error_code visir_spc_sens_stat(const visir_spc_config * pconfig,
+                                          double * psens_median,
                                           double * psens_mean,
                                           double * psens_stdev,
                                           const cpl_table * spc_table)
@@ -583,9 +597,11 @@ static cpl_error_code visir_spc_sens_stat(const visir_spc_phot_config * pconfig,
        - ignore first and last npix_ignore elements */
     /* FIXME: Use cpl_table_select() */
     /* sens_ignore will _not_ be modified */
+    IRPLIB_DIAG_PRAGMA_PUSH_IGN(-Wcast-qual);
     sens_ignore = cpl_vector_wrap( npix - 2*npix_ignore, (double*)
                                cpl_table_get_data_double_const(spc_table,
                                                  "SENSITIVITY") + npix_ignore);
+    IRPLIB_DIAG_PRAGMA_POP;
     skip_if(0);
 
     if (pconfig->plot)
@@ -599,7 +615,7 @@ static cpl_error_code visir_spc_sens_stat(const visir_spc_phot_config * pconfig,
     emis_max = cpl_table_get_column_max(spc_table, "SPC_EMISSIVITY");
     skip_if(0);
 
-    emis_max = emis_min + pconfig->emis_tol * (emis_max - emis_min);
+    emis_max = emis_min + pconfig->phot_emis_tol * (emis_max - emis_min);
 
     iok = 0;
     for (i=0; i < npix - 2*npix_ignore; i++) {
@@ -635,6 +651,7 @@ static cpl_error_code visir_spc_sens_stat(const visir_spc_phot_config * pconfig,
 
     }
 
+    *psens_median =  cpl_vector_get_median(sens_mini);
 
     end_skip;
 
@@ -649,11 +666,13 @@ static cpl_error_code visir_spc_sens_stat(const visir_spc_phot_config * pconfig,
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Append the spectro-phot QC parameters to the supplied list
-  @param    self       The list of properties to be extended
-  @param    star_name  The name of the star, or NULL
-  @param    exptime    The exposure time
-  @param    sens_mean  The mean of the sensitivity (over the wavelenghts)
-  @param    sens_stdev The stdev of the sensitivity (over the wavelenghts)
+  @param    self        The list of properties to be extended
+  @param    star_name   The name of the star, or NULL
+  @param    exptime     The exposure time
+  @param    exectime    The execution time
+  @param    sens_median The median of the sensitivity (over the wavelenghts)
+  @param    sens_mean   The mean of the sensitivity (over the wavelenghts)
+  @param    sens_stdev  The stdev of the sensitivity (over the wavelenghts)
   @return   CPL_ERROR_NONE, or the relevant CPL_ERROR.
 
  */
@@ -661,11 +680,19 @@ static cpl_error_code visir_spc_sens_stat(const visir_spc_phot_config * pconfig,
 static cpl_error_code visir_spc_phot_qc(cpl_propertylist * self,
                                         const char * star_name,
                                         double exptime,
+                                        double exectime,
+                                        double sens_median,
                                         double sens_mean,
                                         double sens_stdev)
 {
 
     bug_if (cpl_propertylist_append_double(self, "ESO QC EXPTIME", exptime));
+    if (exectime > 0) {
+        bug_if (cpl_propertylist_append_double(self, "ESO QC EXECTIME",
+                                               exectime));
+    }
+    bug_if (cpl_propertylist_append_double(self, "ESO QC SENS MEDIAN",
+                                           sens_median));
     bug_if (cpl_propertylist_append_double(self, "ESO QC SENS MEAN", sens_mean));
     bug_if (cpl_propertylist_append_double(self, "ESO QC SENS STDEV",
                                            sens_stdev));
